@@ -1,19 +1,15 @@
-# -*- coding: utf-8 -*-
-
-import py
 import pytest
 
 import sys
 import ssl
 import socket
-import threading
 import datetime
-from concurrent.futures import ThreadPoolExecutor  # type: ignore[import]
-
+from concurrent.futures import ThreadPoolExecutor
 from ipaddress import IPv4Address, IPv6Address, IPv4Network, IPv6Network
+from pathlib import Path
+from typing import Callable, Optional, Union
 
 from cryptography import x509
-from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.serialization import (
     Encoding, PublicFormat, load_pem_private_key)
 
@@ -21,23 +17,18 @@ import OpenSSL.SSL
 import service_identity.pyopenssl  # type: ignore[import]
 
 import trustme
-from trustme import CA, LeafCert
-
-TYPE_CHECKING = False
-if TYPE_CHECKING:  # pragma: no cover
-    from typing import Callable, Optional, Text, Union
-
-    SslSocket = Union[ssl.SSLSocket, OpenSSL.SSL.Connection]
+from trustme import CA, LeafCert, KeyType
 
 
-def _path_length(ca_cert):
-    # type: (x509.Certificate) -> Optional[int]
+SslSocket = Union[ssl.SSLSocket, OpenSSL.SSL.Connection]
+
+
+def _path_length(ca_cert: x509.Certificate) -> Optional[int]:
     bc = ca_cert.extensions.get_extension_for_class(x509.BasicConstraints)
     return bc.value.path_length
 
 
-def assert_is_ca(ca_cert):
-    # type: (x509.Certificate) -> None
+def assert_is_ca(ca_cert: x509.Certificate) -> None:
     bc = ca_cert.extensions.get_extension_for_class(x509.BasicConstraints)
     assert bc.value.ca is True
     assert bc.critical is True
@@ -51,8 +42,7 @@ def assert_is_ca(ca_cert):
         ca_cert.extensions.get_extension_for_class(x509.ExtendedKeyUsage)
 
 
-def assert_is_leaf(leaf_cert):
-    # type: (x509.Certificate) -> None
+def assert_is_leaf(leaf_cert: x509.Certificate) -> None:
     bc = leaf_cert.extensions.get_extension_for_class(x509.BasicConstraints)
     assert bc.value.ca is False
     assert bc.critical is True
@@ -73,26 +63,30 @@ def assert_is_leaf(leaf_cert):
     assert eku.critical is True
 
 
-def test_basics():
-    # type: () -> None
-    ca = CA()
+@pytest.mark.parametrize(
+    "key_type,expected_key_header", [(KeyType.RSA, b"RSA"), (KeyType.ECDSA, b"EC")]
+)
+def test_basics(key_type: KeyType, expected_key_header: bytes) -> None:
+    ca = CA(key_type=key_type)
 
     today = datetime.datetime.today()
 
-    assert b"BEGIN RSA PRIVATE KEY" in ca.private_key_pem.bytes()
+    assert (
+        b"BEGIN " + expected_key_header + b" PRIVATE KEY" in ca.private_key_pem.bytes()
+    )
     assert b"BEGIN CERTIFICATE" in ca.cert_pem.bytes()
 
-    private_key = load_pem_private_key(
-        ca.private_key_pem.bytes(), password=None, backend=default_backend())
+    private_key = load_pem_private_key(ca.private_key_pem.bytes(), password=None)
 
-    ca_cert = x509.load_pem_x509_certificate(
-        ca.cert_pem.bytes(), default_backend())
+    ca_cert = x509.load_pem_x509_certificate(ca.cert_pem.bytes())
     assert ca_cert.not_valid_before <= today <= ca_cert.not_valid_after
 
     public_key1 = private_key.public_key().public_bytes(
-        Encoding.PEM, PublicFormat.PKCS1)
+        Encoding.PEM, PublicFormat.SubjectPublicKeyInfo
+    )
     public_key2 = ca_cert.public_key().public_bytes(
-        Encoding.PEM, PublicFormat.PKCS1)
+        Encoding.PEM, PublicFormat.SubjectPublicKeyInfo
+    )
     assert public_key1 == public_key2
 
     assert ca_cert.issuer == ca_cert.subject
@@ -101,7 +95,9 @@ def test_basics():
     with pytest.raises(ValueError):
         ca.issue_cert()
 
-    server = ca.issue_cert(u"test-1.example.org", u"test-2.example.org")
+    server = ca.issue_cert(
+        "test-1.example.org", "test-2.example.org", key_type=key_type
+    )
 
     assert b"PRIVATE KEY" in server.private_key_pem.bytes()
     assert b"BEGIN CERTIFICATE" in server.cert_chain_pems[0].bytes()
@@ -110,8 +106,7 @@ def test_basics():
     for blob in server.cert_chain_pems:
         assert blob.bytes() in server.private_key_and_cert_chain_pem.bytes()
 
-    server_cert = x509.load_pem_x509_certificate(
-        server.cert_chain_pems[0].bytes(), default_backend())
+    server_cert = x509.load_pem_x509_certificate(server.cert_chain_pems[0].bytes())
 
     assert server_cert.not_valid_before <= today <= server_cert.not_valid_after
     assert server_cert.issuer == ca_cert.subject
@@ -119,20 +114,16 @@ def test_basics():
 
     san = server_cert.extensions.get_extension_for_class(x509.SubjectAlternativeName)
     hostnames = san.value.get_values_for_type(x509.DNSName)
-    assert hostnames == [u"test-1.example.org", u"test-2.example.org"]
+    assert hostnames == ["test-1.example.org", "test-2.example.org"]
 
 
-def test_ca_custom_names():
-    # type: () -> None
+def test_ca_custom_names() -> None:
     ca = CA(
-        organization_name=u'python-trio',
-        organization_unit_name=u'trustme',
+        organization_name='python-trio',
+        organization_unit_name='trustme',
     )
 
-    ca_cert = x509.load_pem_x509_certificate(
-        ca.cert_pem.bytes(),
-        default_backend(),
-    )
+    ca_cert = x509.load_pem_x509_certificate(ca.cert_pem.bytes())
 
     assert {
         'O=python-trio',
@@ -143,19 +134,15 @@ def test_ca_custom_names():
     })
 
 
-def test_issue_cert_custom_names():
-    # type: () -> None
+def test_issue_cert_custom_names() -> None:
     ca = CA()
     leaf_cert = ca.issue_cert(
-        u'example.org',
-        organization_name=u'python-trio',
-        organization_unit_name=u'trustme',
+        'example.org',
+        organization_name='python-trio',
+        organization_unit_name='trustme',
     )
 
-    cert = x509.load_pem_x509_certificate(
-        leaf_cert.cert_chain_pems[0].bytes(),
-        default_backend(),
-    )
+    cert = x509.load_pem_x509_certificate(leaf_cert.cert_chain_pems[0].bytes())
 
     assert {
         'O=python-trio',
@@ -166,57 +153,49 @@ def test_issue_cert_custom_names():
     })
 
 
-def test_issue_cert_custom_not_after():
-    # type: () -> None
-     now = datetime.datetime.now()
-     expires = datetime.datetime(2025, 12, 1, 8, 10, 10)
-     ca = CA()
-
-     leaf_cert = ca.issue_cert(
-         u'example.org',
-         organization_name=u'python-trio',
-         organization_unit_name=u'trustme',
-         not_after=expires,
-     )
-
-     cert = x509.load_pem_x509_certificate(
-         leaf_cert.cert_chain_pems[0].bytes(),
-         default_backend(),
-     )
-
-     for t in ["year", "month", "day", "hour", "minute", "second"]:
-         assert getattr(cert.not_valid_after, t) == getattr(expires, t)
-
-
-def test_intermediate():
-    # type: () -> None
+def test_issue_cert_custom_not_after() -> None:
+    now = datetime.datetime.now()
+    expires = datetime.datetime(2025, 12, 1, 8, 10, 10)
     ca = CA()
-    ca_cert = x509.load_pem_x509_certificate(
-        ca.cert_pem.bytes(), default_backend())
+
+    leaf_cert = ca.issue_cert(
+        "example.org",
+        organization_name="python-trio",
+        organization_unit_name="trustme",
+        not_after=expires,
+    )
+
+    cert = x509.load_pem_x509_certificate(leaf_cert.cert_chain_pems[0].bytes())
+
+    for t in ["year", "month", "day", "hour", "minute", "second"]:
+        assert getattr(cert.not_valid_after, t) == getattr(expires, t)
+
+
+def test_intermediate() -> None:
+    ca = CA()
+    ca_cert = x509.load_pem_x509_certificate(ca.cert_pem.bytes())
     assert_is_ca(ca_cert)
     assert ca_cert.issuer == ca_cert.subject
     assert _path_length(ca_cert) == 9
 
     child_ca = ca.create_child_ca()
-    child_ca_cert = x509.load_pem_x509_certificate(
-        child_ca.cert_pem.bytes(), default_backend())
+    child_ca_cert = x509.load_pem_x509_certificate(child_ca.cert_pem.bytes())
     assert_is_ca(child_ca_cert)
     assert child_ca_cert.issuer == ca_cert.subject
     assert _path_length(child_ca_cert) == 8
 
-    child_server = child_ca.issue_cert(u"test-host.example.org")
+    child_server = child_ca.issue_cert("test-host.example.org")
     assert len(child_server.cert_chain_pems) == 2
     child_server_cert = x509.load_pem_x509_certificate(
-        child_server.cert_chain_pems[0].bytes(), default_backend())
+        child_server.cert_chain_pems[0].bytes()
+    )
     assert child_server_cert.issuer == child_ca_cert.subject
     assert_is_leaf(child_server_cert)
 
 
-def test_path_length():
-    # type: () -> None
+def test_path_length() -> None:
     ca = CA()
-    ca_cert = x509.load_pem_x509_certificate(
-        ca.cert_pem.bytes(), default_backend())
+    ca_cert = x509.load_pem_x509_certificate(ca.cert_pem.bytes())
     assert _path_length(ca_cert) == 9
 
     child_ca = ca
@@ -224,17 +203,15 @@ def test_path_length():
         child_ca = child_ca.create_child_ca()
 
     # Can't create new child CAs anymore
-    child_ca_cert = x509.load_pem_x509_certificate(
-        child_ca.cert_pem.bytes(), default_backend())
+    child_ca_cert = x509.load_pem_x509_certificate(child_ca.cert_pem.bytes())
     assert _path_length(child_ca_cert) == 0
     with pytest.raises(ValueError):
         child_ca.create_child_ca()
 
 
-def test_unrecognized_context_type():
-    # type: () -> None
+def test_unrecognized_context_type() -> None:
     ca = CA()
-    server = ca.issue_cert(u"test-1.example.org")
+    server = ca.issue_cert("test-1.example.org")
 
     with pytest.raises(TypeError):
         ca.configure_trust(None)  # type: ignore[arg-type]
@@ -243,8 +220,7 @@ def test_unrecognized_context_type():
         server.configure_cert(None)  # type: ignore[arg-type]
 
 
-def test_blob(tmpdir):
-    # type: (py.path.local) -> None
+def test_blob(tmp_path: Path) -> None:
     test_data = b"xyzzy"
     b = trustme.Blob(test_data)
 
@@ -254,44 +230,45 @@ def test_blob(tmpdir):
 
     # write_to_path
 
-    b.write_to_path(str(tmpdir / "test1"))
-    with (tmpdir / "test1").open("rb") as f:
+    b.write_to_path(str(tmp_path / "test1"))
+    with (tmp_path / "test1").open("rb") as f:
         assert f.read() == test_data
 
     # append=False overwrites
-    with (tmpdir / "test2").open("wb") as f:
+    with (tmp_path / "test2").open("wb") as f:
         f.write(b"asdf")
-    b.write_to_path(str(tmpdir / "test2"))
-    with (tmpdir / "test2").open("rb") as f:
+    b.write_to_path(str(tmp_path / "test2"))
+    with (tmp_path / "test2").open("rb") as f:
         assert f.read() == test_data
 
     # append=True appends
-    with (tmpdir / "test2").open("wb") as f:
+    with (tmp_path / "test2").open("wb") as f:
         f.write(b"asdf")
-    b.write_to_path(str(tmpdir / "test2"), append=True)
-    with (tmpdir / "test2").open("rb") as f:
+    b.write_to_path(str(tmp_path / "test2"), append=True)
+    with (tmp_path / "test2").open("rb") as f:
         assert f.read() == b"asdf" + test_data
 
     # tempfile
-    with b.tempfile(dir=str(tmpdir)) as path:
-        assert path.startswith(str(tmpdir))
+    with b.tempfile(dir=str(tmp_path)) as path:
+        assert path.startswith(str(tmp_path))
         assert path.endswith(".pem")
         with open(path, "rb") as f:
             assert f.read() == test_data
 
-def test_ca_from_pem(tmpdir):
-    # type: (py.path.local) -> None
+def test_ca_from_pem(tmp_path: Path) -> None:
     ca1 = trustme.CA()
     ca2 = trustme.CA.from_pem(ca1.cert_pem.bytes(), ca1.private_key_pem.bytes())
     assert ca1._certificate == ca2._certificate
     assert ca1.private_key_pem.bytes() == ca2.private_key_pem.bytes()
 
 
-def check_connection_end_to_end(wrap_client, wrap_server):
-    # type: (Callable[[CA, socket.socket, Text], SslSocket], Callable[[LeafCert, socket.socket], SslSocket]) -> None
+def check_connection_end_to_end(
+    wrap_client: Callable[[CA, socket.socket, str], SslSocket],
+    wrap_server: Callable[[LeafCert, socket.socket], SslSocket],
+    key_type: KeyType,
+) -> None:
     # Client side
-    def fake_ssl_client(ca, raw_client_sock, hostname):
-        # type: (CA, socket.socket, Text) -> None
+    def fake_ssl_client(ca: CA, raw_client_sock: socket.socket, hostname: str) -> None:
         try:
             wrapped_client_sock = wrap_client(ca, raw_client_sock, hostname)
             # Send and receive some data to prove the connection is good
@@ -305,8 +282,7 @@ def check_connection_end_to_end(wrap_client, wrap_server):
             raw_client_sock.close()
 
     # Server side
-    def fake_ssl_server(server_cert, raw_server_sock):
-        # type: (LeafCert, socket.socket) -> None
+    def fake_ssl_server(server_cert: LeafCert, raw_server_sock: socket.socket) -> None:
         try:
             wrapped_server_sock = wrap_server(server_cert, raw_server_sock)
             # Prove that we're connected
@@ -319,8 +295,7 @@ def check_connection_end_to_end(wrap_client, wrap_server):
         finally:
             raw_server_sock.close()
 
-    def doit(ca, hostname, server_cert):
-        # type: (CA, Text, LeafCert) -> None
+    def doit(ca: CA, hostname: str, server_cert: LeafCert) -> None:
         # socketpair and ssl don't work together on py2, because... reasons.
         # So we need to do this the hard way.
         listener = socket.socket()
@@ -336,15 +311,15 @@ def check_connection_end_to_end(wrap_client, wrap_server):
             f1.result()
             f2.result()
 
-    ca = CA()
-    intermediate_ca = ca.create_child_ca()
-    hostname = u"my-test-host.example.org"
+    ca = CA(key_type=key_type)
+    intermediate_ca = ca.create_child_ca(key_type=key_type)
+    hostname = "my-test-host.example.org"
 
     # Should work
-    doit(ca, hostname, ca.issue_cert(hostname))
+    doit(ca, hostname, ca.issue_cert(hostname, key_type=key_type))
 
     # Should work
-    doit(ca, hostname, intermediate_ca.issue_cert(hostname))
+    doit(ca, hostname, intermediate_ca.issue_cert(hostname, key_type=key_type))
 
     # To make sure that the above success actually required that the
     # CA and cert logic is all working, make sure that the same code
@@ -352,23 +327,21 @@ def check_connection_end_to_end(wrap_client, wrap_server):
 
     # Bad hostname fails
     with pytest.raises(Exception):
-        doit(ca, u"asdf.example.org", ca.issue_cert(hostname))
+        doit(ca, "asdf.example.org", ca.issue_cert(hostname, key_type=key_type))
 
     # Bad CA fails
     bad_ca = CA()
     with pytest.raises(Exception):
-        doit(bad_ca, hostname, ca.issue_cert(hostname))
+        doit(bad_ca, hostname, ca.issue_cert(hostname, key_type=key_type))
 
 
-def test_stdlib_end_to_end():
-    # type: () -> None
-    def wrap_client(ca, raw_client_sock, hostname):
-        # type: (CA, socket.socket, Text) -> ssl.SSLSocket
+@pytest.mark.parametrize("key_type", [KeyType.RSA, KeyType.ECDSA])
+def test_stdlib_end_to_end(key_type: KeyType) -> None:
+    def wrap_client(ca: CA, raw_client_sock: socket.socket, hostname: str) -> ssl.SSLSocket:
         ctx = ssl.create_default_context()
         ca.configure_trust(ctx)
-        # Type ignore for Python 2: wants str, got unicode, but I guess unicode also works.
         wrapped_client_sock = ctx.wrap_socket(
-            raw_client_sock, server_hostname=hostname)  # type: ignore[arg-type]
+            raw_client_sock, server_hostname=hostname)
         print("Client got server cert:", wrapped_client_sock.getpeercert())
         peercert = wrapped_client_sock.getpeercert()
         assert peercert is not None
@@ -376,8 +349,7 @@ def test_stdlib_end_to_end():
         assert san == (("DNS", "my-test-host.example.org"),)
         return wrapped_client_sock
 
-    def wrap_server(server_cert, raw_server_sock):
-        # type: (LeafCert, socket.socket) -> ssl.SSLSocket
+    def wrap_server(server_cert: LeafCert, raw_server_sock: socket.socket) -> ssl.SSLSocket:
         ctx = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
         server_cert.configure_cert(ctx)
         wrapped_server_sock = ctx.wrap_socket(
@@ -385,13 +357,12 @@ def test_stdlib_end_to_end():
         print("server encrypted with:", wrapped_server_sock.cipher())
         return wrapped_server_sock
 
-    check_connection_end_to_end(wrap_client, wrap_server)
+    check_connection_end_to_end(wrap_client, wrap_server, key_type)
 
 
-def test_pyopenssl_end_to_end():
-    # type: () -> None
-    def wrap_client(ca, raw_client_sock, hostname):
-        # type: (CA, socket.socket, Text) -> OpenSSL.SSL.Connection
+@pytest.mark.parametrize("key_type", [KeyType.RSA, KeyType.ECDSA])
+def test_pyopenssl_end_to_end(key_type: KeyType) -> None:
+    def wrap_client(ca: CA, raw_client_sock: socket.socket, hostname: str) -> OpenSSL.SSL.Connection:
         # Cribbed from example at
         #   https://service-identity.readthedocs.io/en/stable/api.html#service_identity.pyopenssl.verify_hostname
         ctx = OpenSSL.SSL.Context(OpenSSL.SSL.SSLv23_METHOD)
@@ -404,8 +375,7 @@ def test_pyopenssl_end_to_end():
         service_identity.pyopenssl.verify_hostname(conn, hostname)
         return conn
 
-    def wrap_server(server_cert, raw_server_sock):
-        # type: (LeafCert, socket.socket) -> OpenSSL.SSL.Connection
+    def wrap_server(server_cert: LeafCert, raw_server_sock: socket.socket) -> OpenSSL.SSL.Connection:
         ctx = OpenSSL.SSL.Context(OpenSSL.SSL.SSLv23_METHOD)
         server_cert.configure_cert(ctx)
 
@@ -414,11 +384,10 @@ def test_pyopenssl_end_to_end():
         conn.do_handshake()
         return conn
 
-    check_connection_end_to_end(wrap_client, wrap_server)
+    check_connection_end_to_end(wrap_client, wrap_server, key_type)
 
 
-def test_identity_variants():
-    # type: () -> None
+def test_identity_variants() -> None:
     ca = CA()
 
     for bad in [b"example.org", bytearray(b"example.org"), 123]:
@@ -427,50 +396,50 @@ def test_identity_variants():
 
     cases = {
         # Traditional ascii hostname
-        u"example.org": x509.DNSName(u"example.org"),
+        "example.org": x509.DNSName("example.org"),
 
         # Wildcard
-        u"*.example.org": x509.DNSName(u"*.example.org"),
+        "*.example.org": x509.DNSName("*.example.org"),
 
         # IDN
-        u"éxamplë.org": x509.DNSName(u"xn--xampl-9rat.org"),
-        u"xn--xampl-9rat.org": x509.DNSName(u"xn--xampl-9rat.org"),
+        "éxamplë.org": x509.DNSName("xn--xampl-9rat.org"),
+        "xn--xampl-9rat.org": x509.DNSName("xn--xampl-9rat.org"),
 
         # IDN + wildcard
-        u"*.éxamplë.org": x509.DNSName(u"*.xn--xampl-9rat.org"),
-        u"*.xn--xampl-9rat.org": x509.DNSName(u"*.xn--xampl-9rat.org"),
+        "*.éxamplë.org": x509.DNSName("*.xn--xampl-9rat.org"),
+        "*.xn--xampl-9rat.org": x509.DNSName("*.xn--xampl-9rat.org"),
 
         # IDN that acts differently in IDNA-2003 vs IDNA-2008
-        u"faß.de": x509.DNSName(u"xn--fa-hia.de"),
-        u"xn--fa-hia.de": x509.DNSName(u"xn--fa-hia.de"),
+        "faß.de": x509.DNSName("xn--fa-hia.de"),
+        "xn--fa-hia.de": x509.DNSName("xn--fa-hia.de"),
 
         # IDN with non-permissable character (uppercase K)
         # (example taken from idna package docs)
-        u"Königsgäßchen.de": x509.DNSName(u"xn--knigsgchen-b4a3dun.de"),
+        "Königsgäßchen.de": x509.DNSName("xn--knigsgchen-b4a3dun.de"),
 
         # IP addresses
-        u"127.0.0.1": x509.IPAddress(IPv4Address(u"127.0.0.1")),
-        u"::1": x509.IPAddress(IPv6Address(u"::1")),
+        "127.0.0.1": x509.IPAddress(IPv4Address("127.0.0.1")),
+        "::1": x509.IPAddress(IPv6Address("::1")),
         # Check normalization
-        u"0000::1": x509.IPAddress(IPv6Address(u"::1")),
+        "0000::1": x509.IPAddress(IPv6Address("::1")),
 
         # IP networks
-        u"127.0.0.0/24": x509.IPAddress(IPv4Network(u"127.0.0.0/24")),
-        u"2001::/16": x509.IPAddress(IPv6Network(u"2001::/16")),
+        "127.0.0.0/24": x509.IPAddress(IPv4Network("127.0.0.0/24")),
+        "2001::/16": x509.IPAddress(IPv6Network("2001::/16")),
         # Check normalization
-        u"2001:0000::/16": x509.IPAddress(IPv6Network(u"2001::/16")),
+        "2001:0000::/16": x509.IPAddress(IPv6Network("2001::/16")),
 
         # Email address
-        u"example@example.com": x509.RFC822Name(u"example@example.com"),
+        "example@example.com": x509.RFC822Name("example@example.com"),
     }
 
     for hostname, expected in cases.items():
         # Can't repr the got or expected values here, at least until
         # cryptography v2.1 is out, because in v2.0 on py2, DNSName.__repr__
         # blows up on IDNs.
-        print("testing: {!r}".format(hostname))
+        print(f"testing: {hostname!r}")
         pem = ca.issue_cert(hostname).cert_chain_pems[0].bytes()
-        cert = x509.load_pem_x509_certificate(pem, default_backend())
+        cert = x509.load_pem_x509_certificate(pem)
         san = cert.extensions.get_extension_for_class(
             x509.SubjectAlternativeName
         )
@@ -479,50 +448,43 @@ def test_identity_variants():
         assert got == expected
 
 
-def test_backcompat():
-    # type: () -> None
+def test_backcompat() -> None:
     ca = CA()
     # We can still use the old name
-    ca.issue_server_cert(u"example.com")
+    ca.issue_server_cert("example.com")
 
 
-def test_CN():
-    # type: () -> None
+def test_CN() -> None:
     ca = CA()
 
-    # Since we have to emulate kwonly args here, I guess we should test the
-    # emulation logic
-    with pytest.raises(TypeError):
-        ca.issue_cert(comon_nam=u"wrong kwarg name")  # type: ignore[call-arg]
-
-    # Must be unicode
+    # Must be str
     with pytest.raises(TypeError):
         ca.issue_cert(common_name=b"bad kwarg value")  # type: ignore[arg-type]
 
     # Default is no common name
-    pem = ca.issue_cert(u"example.com").cert_chain_pems[0].bytes()
-    cert = x509.load_pem_x509_certificate(pem, default_backend())
+    pem = ca.issue_cert("example.com").cert_chain_pems[0].bytes()
+    cert = x509.load_pem_x509_certificate(pem)
     common_names = cert.subject.get_attributes_for_oid(
         x509.oid.NameOID.COMMON_NAME
     )
     assert common_names == []
 
     # Common name on its own is valid
-    pem = ca.issue_cert(common_name=u"woo").cert_chain_pems[0].bytes()
-    cert = x509.load_pem_x509_certificate(pem, default_backend())
+    pem = ca.issue_cert(common_name="woo").cert_chain_pems[0].bytes()
+    cert = x509.load_pem_x509_certificate(pem)
     common_names = cert.subject.get_attributes_for_oid(
         x509.oid.NameOID.COMMON_NAME
     )
-    assert common_names[0].value == u"woo"
+    assert common_names[0].value == "woo"
 
     # Common name + SAN
-    pem = ca.issue_cert(u"example.com", common_name=u"woo").cert_chain_pems[0].bytes()
-    cert = x509.load_pem_x509_certificate(pem, default_backend())
+    pem = ca.issue_cert("example.com", common_name="woo").cert_chain_pems[0].bytes()
+    cert = x509.load_pem_x509_certificate(pem)
     san = cert.extensions.get_extension_for_class(
         x509.SubjectAlternativeName
     )
-    assert list(san.value)[0] == x509.DNSName(u"example.com")
+    assert list(san.value)[0] == x509.DNSName("example.com")
     common_names = cert.subject.get_attributes_for_oid(
         x509.oid.NameOID.COMMON_NAME
     )
-    assert common_names[0].value == u"woo"
+    assert common_names[0].value == "woo"
